'use strict';

import { regularPolygonVectorsFromEdgeCount, verticesFromVectors, vectorFromXY, vertexTranslatedByVector, rotate } from './geometry.js';
import { getTileRenderer, getPathRenderer, visualize } from './visualizer.js';
import { squareTessellation, triangleTessellation, squareTriangleTessellation } from './tessellation.js';
import { asIndexable, map, toArray } from './functional.js';
import { mazeFromTessellationWithinBoundingVector } from './maze.js';

require('./styles.scss');

const mazeBounds = vectorFromXY(30, 20);

function generateMazeFromTessellation(tessellation) {
	let maze = mazeFromTessellationWithinBoundingVector(tessellation, mazeBounds);
	let lines = toArray(maze.lines);
	let startingVertex = asIndexable(lines[0])(0);
	let unitVector = asIndexable(tessellation.startingShape)(0);
	let leadInVector = rotate(unitVector, Math.PI / 4 * 5);
	let leadInVertex = vertexTranslatedByVector(startingVertex, leadInVector);
	let renderers = map(lines, (line, index) => getPathRenderer(index == 0 ? [ leadInVertex, ...line ] : line));
	let markup = visualize(renderers);
	return markup;
}

document.getElementById('square-maze').innerHTML = generateMazeFromTessellation(squareTessellation);
document.getElementById('triangle-maze').innerHTML = generateMazeFromTessellation(triangleTessellation);
document.getElementById('square-triangle-maze').innerHTML = generateMazeFromTessellation(squareTriangleTessellation);