'use strict';

/*
 * Functions to support pure mathematical operations.
 * 
 */

import { asReiterable, reduce } from './functional.js';

// Add up.

function sum(iterator) {
	return reduce(iterator, (accumulator, value) => accumulator + value, 0);
}

// A (potentially infinite) range of numbers.

function range(from, to, step) {
	function* generator() {
		if (arguments.length === 1) {
			to = from;
			from = 0;
		}
		if (arguments.length < 3) {
			step = 1;
		}
		for (let i = from; i <= to; i += step) {
			yield i;
		}
	};
	return asReiterable(generator);
}

// Round to help avoid problems introduced by floating-point imprecisions.

function round(n, decimalPoints = 3) {
	let scale = Math.pow(10, decimalPoints);
	return Math.round(n * scale) / scale;
}

export {
	range,
	round,
	sum
};