'use strict';

/*
 * Playing around with testing framework ideas. I could have used a library, but where's the fun in that?
 * 
 */

function report(...args) {
	console.log.apply(console.window, args);
}

function reportFailure(testName, e) {
	report(`${testName} failed:`, e.stack);
}

function test(suite) {
	let passedCount = 0;
	let failedCount = 0;
	for (let testName in suite) {
		if (suite.hasOwnProperty(testName)) {
			let test = suite[testName];
			try {
				test();
				++passedCount;
			}
			catch (e) {
				++failedCount;
				reportFailure(testName, e);
			}
		}
	}
	if (failedCount == 0) {
		report(`All ${passedCount} tests passed.`);
	}
	else {
		report(`${failedCount} of ${failedCount + passedCount} tests failed!`);			
	}
}

function isPrimitive(value) {
	return value !== Object(value);
}

function assertLength(object, expectedLength) {
	if (!object) {
		throw Error('Object is null.');
	}
	if (object.length == null) {
		throw Error('Object has no length property.');		
	}
	if (object.length != expectedLength) {
		throw Error(`Object has length ${object.length} instead of expected length ${expectedLength}:\n${object}`);
	}
}

function assertEqual(actual, expected) {
	if (isPrimitive(expected)) {
		if (actual !== expected) {
			throw Error(`Expected primitive ${expected} but got ${actual}.`);
		}
	}
	else {
		for (let member in expected) {
			if (expected.hasOwnProperty(member)) {
				if (!(member in actual)) {
					throw Error(`Expected members ${member} not found in object ${actual}.`);
				}
				try {
					assertEqual(actual[member], expected[member]);
				}
				catch (e) {
					throw Error(`Members ${member} not equal:\n${e.message}`);
				}
			}
		}
	}
}

function assertThat(value) {
	return {
		hasLength: length => assertLength(value, length),
		atIndex: index => {
			let item = Array.isArray(value) ? value[index] : value(index);
			return {
				isEqualTo: expected => {
					try {
						assertEqual(item, expected)
					}
					catch (e) {
						console.log('Test failure value:');
						console.log([...value]);//TODO
						throw e;
					}
				}
			}
		},
		isEqualTo: expected => {
			try {
				assertEqual(value, expected)
			}
			catch (e) {
				console.log('Test failure value:');
				console.log([...value]);//TODO
				throw e;
			}
		}		
	};
}

export {
	test,
	assertThat	// TODO assertions.js or another file?
};