'use strict';

const path = require('path');
const UglifyJsPlugin = require('webpack').optimize.UglifyJsPlugin;
const ProvidePlugin = require('webpack').ProvidePlugin;
const HtmlWebpackPlugin = require('html-webpack-plugin');
const ExtractTextPlugin = require("extract-text-webpack-plugin");

module.exports = {
	context: __dirname,
	resolve: {
		root: path.join(__dirname, 'src'),
	},
	module: {
		preLoaders: [
			{
				test: /\.ejs(\..+)?$/,
				loader: 'ejs'
			}
		],
		loaders: [
			{
				test: /\.js$/,
				loader: 'babel-loader',
				exclude: /node_modules/,
				query: {
//					plugins: [ "babel-plugin-transform-es2015-modules-commonjs" ]		// Swap below line to keep ES6 with imports converted to requires.
					presets: [ 'es2015' ]
				}
			}, {
				test: /\.scss$/,
				loader: 'style!css!sass'
			}, {
				test: /\.css$/,
				loader: 'style!css'
			}
		]
	},
	resolveLoader: {
		modulesDirectories: [
			`${__dirname}/node_modules`
		]
	},
	entry: {
		app: [
			'babel-polyfill',
			'./src/app.js',
			'./src/geometry.tests.js'	// TODO Make a conditional build.
		]
	},
	output: {
		filename: '[name].bundle.js',
		path: './built',
		publicPath: '/'
	},
	plugins: [ // Order is significant.
		new HtmlWebpackPlugin({
      filename: 'index.html',
      template: 'src/index.ejs.html',
			inject: false
    }),
		new UglifyJsPlugin()
	],
	devServer: {
		port: 8082,
		contentBase: './built',
		historyApiFallback: {
			index: '/error/404.html',
			rewrites: [
				{
					from: '/$',
					to: '/index.html'
				}
			]
		}
	}
};